package net.knarcraft.paidsigns.command;

import net.knarcraft.paidsigns.utility.TabCompleteHelper;
import org.bukkit.Bukkit;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.permissions.Permission;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringJoiner;

/**
 * The tab completer for the add paid sign command
 */
public class AddTabCompleter extends TokenizedTabCompleter {

    private static List<String> names;
    private static List<String> costs;
    private static List<String> plugins;
    private static Map<String, List<String>> permissions;
    private static List<String> options;
    private static List<String> booleans;

    @Nullable
    @Override
    public List<String> onTabComplete(@NotNull CommandSender sender, @NotNull Command command, @NotNull String alias,
                                      @NotNull String[] args) {
        super.onTabComplete(sender, command, alias, args);
        if (names == null) {
            initializeValues();
            loadAvailablePermissions();
        }

        if (argumentSize == 1) {
            return TabCompleteHelper.filterMatchingStartsWith(names, arguments.get(0));
        } else if (argumentSize == 2) {
            return TabCompleteHelper.filterMatchingStartsWith(costs, arguments.get(1));
        } else if (argumentSize == 3) {
            return tabCompletePermission(arguments.get(arguments.size() - 1));
        } else if (argumentSize == 4) {
            return TabCompleteHelper.filterMatchingStartsWith(options, arguments.get(3));
        } else if (argumentSize == 5) {
            return TabCompleteHelper.filterMatchingStartsWith(options, arguments.get(4));
        } else if (argumentSize == 6) {
            return TabCompleteHelper.filterMatchingStartsWith(booleans, arguments.get(5));
        }
        return new ArrayList<>();
    }

    /**
     * Gets the tab complete value for the permission typed
     *
     * @param typedNode <p>The full permission node typed by the player</p>
     * @return <p>All known valid auto-complete options</p>
     */
    private List<String> tabCompletePermission(String typedNode) {
        List<String> output;
        if (typedNode.contains(".")) {
            List<String> matchingPermissions = permissions.get(typedNode.substring(0, typedNode.lastIndexOf(".")));
            if (matchingPermissions == null) {
                output = new ArrayList<>();
            } else {
                //Filter by the typed text
                output = filterMatching(matchingPermissions, typedNode);
            }
        } else {
            output = plugins;
        }

        //Add previous permissions in the comma-separated lists as a prefix
        return output;
    }

    /**
     * Find completable strings which match the text typed by the command's sender
     *
     * @param values    <p>The values to filter</p>
     * @param typedText <p>The text the player has started typing</p>
     * @return <p>The given string values which start with the player's typed text</p>
     */
    private List<String> filterMatching(List<String> values, String typedText) {
        List<String> configValues = new ArrayList<>();
        for (String value : values) {
            if (value.toLowerCase().startsWith(typedText.toLowerCase())) {
                configValues.add(value);
            }
        }
        return configValues;
    }

    /**
     * Loads all permissions available from bukkit plugins
     */
    private static void loadAvailablePermissions() {
        plugins = new ArrayList<>();
        permissions = new HashMap<>();

        for (Permission permission : Bukkit.getPluginManager().getPermissions()) {
            loadPermission(permission.getName());
        }
    }

    /**
     * Loads a given permission into the proper lists and maps
     *
     * @param permissionName <p>The permission to load</p>
     */
    private static void loadPermission(String permissionName) {
        String[] permissionParts = permissionName.split("\\.");
        if (permissionParts.length == 1 && !plugins.contains(permissionParts[0])) {
            plugins.add(permissionParts[0]);
        } else if (permissionParts.length > 1) {
            StringJoiner pathJoiner = new StringJoiner(".");
            for (int j = 0; j < permissionParts.length - 1; j++) {
                pathJoiner.add(permissionParts[j]);
            }
            String path = pathJoiner.toString();
            List<String> permissionList = permissions.computeIfAbsent(path, k -> new ArrayList<>());
            permissionList.add(permissionName);

            loadPermission(path);
        }
    }

    /**
     * Initializes the values available for tab completion
     */
    private static void initializeValues() {
        names = TabCompleteHelper.getExamplePaidSignNames();
        costs = TabCompleteHelper.getCosts();
        options = TabCompleteHelper.getOptionStates();
        booleans = TabCompleteHelper.getBooleans();
    }

}
